﻿-- Author: Crashkurs

----------------------------------
--	Creating Lib-Instances		--
----------------------------------

local BC = AceLibrary("Babble-Class-2.2");
local L = AceLibrary("AceLocale-2.2"):new("CWFu");
local Dewdrop = AceLibrary("Dewdrop-2.0");
local tablet = AceLibrary("Tablet-2.0");

----------------------------------
--		Library Mixins			--
----------------------------------

CWFu = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0", "AceDB-2.0", "FuBarPlugin-2.0", "AceHook-2.1", "WhoLib-1.0");

----------------------------------
--	Register DB / DB-Defaults	--
----------------------------------

CWFu.hasIcon = true;
CWFu.clickableTooltip = true;

CWFu:RegisterDB("FuBar_ChannelWatchFuDB");

CWFu:RegisterDefaults('realm', {
		mychars = {},
		notes = {}
		}
	);

CWFu:RegisterDefaults('profile', {
		channellist = {},
		orderlist = {},
		zone = true,
		level = true,
		notes = false,
		guild = false,
		group = true
		}
	);
		
function CWFu:OnInitialize()
	CWFu:RegisterEvent("CHAT_MSG_CHANNEL_NOTICE");
	CWFu:RegisterEvent("CHAT_MSG_CHANNEL_JOIN");
	CWFu:RegisterEvent("CHAT_MSG_CHANNEL_LEAVE");
	--CWFu:RegisterEvent("CHAT_MSG_CHANNEL_LIST");
	--DEFAULT_CHAT_FRAME:UnregisterEvent("CHAT_MSG_CHANNEL_LIST");
	CWFu.Channel = {};
	CWFu.ScanChannel ={};
	CWFu.QueueNum = 0;
	CWFu.Member = {};
	CWFu.OutList = {};
	CWFu.QueuedChannelInfo = {};
	CWFu.isScanning = false;
	CWFu.lastUpdate = 0;
end

----------------------------------
--		Basic Functions			--
----------------------------------

function CWFu:OnTextUpdate()
	local TText = "";
	
	if (CWFu.InChannels()) then
		for _,cname in ipairs(CWFu.OutList) do
			local chan = CWFu.Channel[cname[1]];
			if (chan.watch) then
				TText = TText .. #chan.Members .. "/";
			end --if
		end --for n
		TText = string.sub(TText, 0, -2);
		self:SetText(TText);
	else
		self:SetText(self:GetTitle());
	end --if
end

function CWFu.InChannels()
	local var = false;
	table.foreach(CWFu.Channel, function() var = true return var end);
	return var;
end

function CWFu:OnClick()
	CWFu:ChannelsUpdate();
end

----------------------------------
--		Creating the Menu		--
----------------------------------

function CWFu:OnMenuRequest(level, value)
local already_listed;

	if (level == 1) then
		Dewdrop:AddLine(
			'text', L["CHANNELS"],
			'hasArrow', true,
			'value', "CHANNELS");
		Dewdrop:AddLine(
			'text', L["TOOLTIP"],
			'hasArrow', true,
			'value', "TOOLTIP");
		Dewdrop:AddLine();
	
	elseif (level == 2) then
		if (value == "CHANNELS") then
			for _,n in pairs(CWFu.Channel) do
				Dewdrop:AddLine(
					'text', n.name,
					'hasArrow', true,
					'value', "CHANNEL_" .. n.name);
			end --for n
			
			for _,n in ipairs(self.db.profile.channellist) do
				already_listed = false;
				for _,m in pairs(CWFu.Channel) do
					if (string.upper(n) == string.upper(m.name)) then
						already_listed = true;
					end --if
				end --for m
				
				if (not already_listed) then
					Dewdrop:AddLine(
						'text', n,
						'hasArrow', true,
						'value', "CHANNEL_" .. n);
				end --if
			end --for n
			
			Dewdrop:AddLine();
			Dewdrop:AddLine(
				'text', L["JOIN"],
				'hasArrow', true,
				'hasEditBox', true,
				'editBoxFunc', function(text)
								local exists = false
								for _,n in ipairs(self.db.profile.channellist) do
									if (string.upper(n) == string.upper(text)) then
										exists = true
									end --if
								end --for n
								if (not exists) then
									for _,n in pairs(CWFu.Channel) do
										if (string.upper(n.name) == string.upper(text)) then
											exists = true
										end --if
									end --for n
									if (not exists) then
										table.insert(self.db.profile.channellist, text)
										JoinChannelByName(text)
										self:Print(L["JOINED_C"] .. ": " .. text);
										self:OpenMenu()
									else
										--ausgabe existiert bereits
									end --if
								else
									--ausgabe existiert bereits
								end --if
							end);
		elseif (value == "TOOLTIP") then
			Dewdrop:AddLine(
				'text', L["LEVEL"],
				'func', function()
							self.db.profile.level = not self.db.profile.level
						end,
				'checked', self.db.profile.level);
			Dewdrop:AddLine(
				'text', L["GROUP"],
				'func', function()
							self.db.profile.group = not self.db.profile.group
						end,
				'checked', self.db.profile.group);
			Dewdrop:AddLine(
				'text', L["GUILD"],
				'func', function()
							self.db.profile.guild = not self.db.profile.guild
						end,
				'checked', self.db.profile.guild);
			Dewdrop:AddLine(
				'text', L["ZONE"],
				'func', function()
							self.db.profile.zone = not self.db.profile.zone
						end,
				'checked', self.db.profile.zone);
			Dewdrop:AddLine(
				'text', L["NOTES"],
				'func', function()
							self.db.profile.notes = not self.db.profile.notes
						end,
				'checked', self.db.profile.notes);
		end --if
	
	elseif (level == 3) then
		if (string.sub(value, 1, 8) == "CHANNEL_") then
			local channelname = string.upper(string.sub(value, 9));
			local temp_n;
			local is_active;

			if (CWFu.Channel[channelname]) then
				is_active = true;
			end --if
			
			for n,m in ipairs(self.db.profile.channellist) do
				if (string.upper(m) == string.upper(channelname)) then
					temp_n = n;
				end --if
			end --for n

			Dewdrop:AddLine(
				'text', L["JOIN"],
				'checked', is_active,
				'func', function()
							if (is_active) then
								LeaveChannelByName(channelname)
								self:Print(L["LEFT_C"] .. ": " .. channelname)
							else
								JoinChannelByName(channelname)
								self:Print(L["JOINED_C"] .. ": " .. channelname)
							end --if
							self:OpenMenu()
							--AceLibrary("Dewdrop-2.0"):Refresh()
						end);
			
			if (is_active) then
				Dewdrop:AddLine(
					'text', L["WATCH"],
					'checked', CWFu.Channel[channelname].watch,
					'func', function()
								if (CWFu.Channel[channelname].watch) then
									table.remove(self.db.profile.channellist, temp_n)
								else
									table.insert(self.db.profile.channellist, channelname)
									CWFu:GetMembers(channelname)
								end --if
								CWFu.Channel[channelname].watch = not CWFu.Channel[channelname].watch
								self:OpenMenu()
								CWFu:Update()
							end);
			else
				Dewdrop:AddLine(
					'text', L["WATCH"],
					'checked', true,
					'func', function()
								table.remove(self.db.profile.channellist, temp_n)
								self:OpenMenu()
								CWFu:Update()
								--AceLibrary("Dewdrop-2.0"):Refresh()
							end);				
			end --if
		end --if
	end --if
end

----------------------------------
--	Creating the Tooltip		--
----------------------------------

function CWFu:OnDataUpdate()

	if(not CWFu.InChannels()) then
		CWFu:ChannelsUpdate();
	end--if

	CWFu.OutList = {};
	
	--if (not CWFu.db.profile.orderlist) then
		local x = 0;
		table.foreach(CWFu.Channel, function(a,b) x=x+1 CWFu.OutList[x] = {a, b.number} end);
		table.sort(CWFu.OutList, function(a,b) return a[2]<b[2] end);
	--end --if
end

function CWFu:OnTooltipUpdate()
	
	if (not CWFu:IsEventScheduled("MembersUpdate")) then
		CWFu:ScheduleEvent("MembersUpdate", CWFu.MembersUpdate, 300, CWFu)
	end --if
	
	local cols = {};
	table.insert(cols, L["NAME"]);
	--if (self.db.profile.level) then
	--	table.insert(cols, L["LEVEL"]);
	--end --if
	if (self.db.profile.guild) then
		table.insert(cols, L["GUILD"]);
	end --if
	if (self.db.profile.zone) then
		table.insert(cols, L["ZONE"]);
	end --if
	if (self.db.profile.notes) then
		table.insert(cols, L["NOTES"]);
	end --if

	local header = {};
	
	if (self.db.profile.group) then
		header['hasCheck'] = true;
	end --if
	
	for n,m in ipairs(cols) do
		if (n==1) then
			header['text'] = m;
			header['justify'] = "LEFT";
		else
			header['text'..n] = m;
			header['justify'..n] = "CENTER";
		end --if
	end --for n
	
	for _,cname in ipairs(CWFu.OutList) do
		local chan = CWFu.Channel[cname[1]];
		if (chan.watch) then
			local cat = tablet:AddCategory(
				'text', string.upper(chan.name) .. " (" .. #chan.Members .. ")",
				'size', 16,
				'columns', #cols);

			cat:AddLine(header);
			
			local needRecycle = false;
			--gehe über alle Members und füge hinzu wenn mitglied vom aktuellen channel
			for n,m in ipairs(chan.Members) do
				if (CWFu.Member[m]) then
					local line = {};
					local colcount = 1;
					local cc = {BC:GetColor(CWFu.Member[m].class)};
					
					--[[if (CWFu.UnitInGroup(CWFu.Member[m].name)) then
						CWFu.Member[m].level = UnitLevel(CWFu.Member[m].name);
						CWFu.Member[m].zone = user.Zone;
						CWFu.Member[m].guild, _, _ = GetGuildInfo(CWFu.Member[m].name);
						CWFu.Member[m].class = UnitClass(CWFu.Member[m].name);
					end --if]]
					
					if (self.db.profile.level) then
						line['text'] = CWFu.Member[m].name .. " (" .. CWFu.Member[m].level .. ")";
					else
						line['text'] = CWFu.Member[m].name;
					end --if
					line['textR'] = cc[1];
					line['textG'] = cc[2];
					line['textB'] = cc[3];
					
					if (self.db.profile.group) then
						line['hasCheck'] = true;
						line['checked'] = CWFu.UnitInGroup(m);
					end --if
					
					if (self.db.profile.guild) then
						colcount = colcount + 1;
						line['text'..colcount] = CWFu.Member[m].guild;
					end --if
					if (self.db.profile.zone) then
						colcount = colcount + 1;
						line['text'..colcount] = CWFu.Member[m].zone;
					end --if
					if (self.db.profile.notes) then
						colcount = colcount + 1;
						line['text'..colcount] = CWFu.Member[m].notes;
					end --if
					
					line['func'] = 'OnMemberClick';
					line['arg1'] = CWFu;
					line['arg2'] = CWFu.Member[m].name;
					--table.insert(AllLines, line);
					cat:AddLine(line);
				else
					CWFu.Channel[cname[1]].Members[n] = "CWFu_Recycle";
					needRecycle = true;
					--self:Print("Lösche: " .. m);
				end
			end --for m
			
			if (needRecycle) then
				CWFu.Channel[cname[1]].Members = CWFu.TableRecycle(CWFu.Channel[cname[1]].Members);
				--self:Print("Recycled");
			end --if
		end --if
	end --for chan
	
	tablet:SetHint(L["HINT"]);
end

function CWFu.UnitInGroup(name)
	if (GetNumRaidMembers() == 0) then
		return UnitInParty(name);
	else
		return UnitInRaid(name);
	end --if
end

function CWFu:GetMembers(channel)
	--ListChannelByName(channel)
	--wird im chatfenster ausgegeben

	if (not CWFu.isScanning) then
		--self:Print("activating for: " .. channel);
		CWFu:ActivateScanning();
		CWFu.isScanning = true;
		CWFu.QueueNum = 0;
		CWFu.ScanChannel[channel] = true;
		ListChannelByName(channel);
		CWFu:ScheduleEvent("DeactivateScanning", CWFu.DeactivateScanning, 1, CWFu);
		CWFu:ScheduleEvent("FinishScanning" .. channel, CWFu.FinishedScanning, 1, CWFu, channel);
	else
		CWFu.QueueNum = CWFu.QueueNum + 1;
		CWFu:ScheduleEvent("QueueChannel" .. channel, CWFu.GetMembers, CWFu.QueueNum * 2, CWFu, channel);
		if (CWFu:IsEventScheduled("DeactivateScanning")) then
			CWFu:CancelScheduledEvent("DeactivateScanning");
		end --if
	end --if
end

function CWFu:FinishedScanning(channel)
	CWFu.isScanning = false;
	CWFu.ScanChannel[channel] = false;
end

function CWFu:ActivateScanning()
	if (not CWFu:IsEventRegistered("CHAT_MSG_CHANNEL_LIST")) then
		CWFu:RegisterEvent("CHAT_MSG_CHANNEL_LIST");
		DEFAULT_CHAT_FRAME:UnregisterEvent("CHAT_MSG_CHANNEL_LIST");
	end --if
end

function CWFu:DeactivateScanning()
	if (CWFu:IsEventRegistered("CHAT_MSG_CHANNEL_LIST")) then
		CWFu:UnregisterEvent("CHAT_MSG_CHANNEL_LIST");
		DEFAULT_CHAT_FRAME:RegisterEvent("CHAT_MSG_CHANNEL_LIST");
	end --if
	table.foreach(CWFu.QueuedChannelInfo, function(a,b) ListChannelByName(b) end);
	CWFu.QueuedChannelInfo = {};
end

function CWFu:OnMemberClick(name)
	--self:Print(name);
	if IsAltKeyDown() then
		InviteUnit(name);
	elseif IsShiftKeyDown() then
		SendWho('n-\"'..name..'\"');
		CWFu:UserInfo(name, {callback = 'UserDataReturned', timeout = 0, flags = self.WHOLIB_FLAG_ALLWAYS_CALLBACK});
	elseif IsControlKeyDown() then
		--noch nichts
	else
		SetItemRef("player:"..name, "|Hplayer:"..name.."|h["..name.."|h", "LeftButton");
	end --if

end

function CWFu:MembersUpdate()
	local TimeToWait = 120;
	for _,m in pairs(CWFu.Member) do
		if (not CWFu.UnitInGroup(m.name)) then
			CWFu:UserInfo(m.name, {callback = 'UserDataReturned', timeout = 2, flags = self.WHOLIB_FLAG_ALLWAYS_CALLBACK, queue = self.WHOLIB_QUEUE_SCANNING});
			TimeToWait = TimeToWait + 10;
		end --if
	end --for
	CWFu:ScheduleEvent("MembersUpdate", CWFu.MembersUpdate, TimeToWait, CWFu);
end

function CWFu:EditNote(name)

end

--function CWFu:Who(name)
--	CWFu:UserInfo(name, {callback = 'UserDataReturned', timeout = 0, flags = self.WHOLIB_FLAG_ALLWAYS_CALLBACK});
--end

function CWFu:UserDataReturned(user, zeit)
	if (CWFu.Member[user.Name]) then
		if (user.Online) then
			CWFu.Member[user.Name].name = user.Name;
			CWFu.Member[user.Name].level = user.Level;
			CWFu.Member[user.Name].zone = user.Zone;
			CWFu.Member[user.Name].guild = user.Guild;
			CWFu.Member[user.Name].class = user.Class;
			CWFu:UpdateTooltip();
		else --if (user.Online == false) then
			--self:Print("Member gelöscht: " .. user.Name);
			CWFu.Member[user.Name] = "CWFu_Recycle";
			CWFu.Member = CWFu.TableRecycle(CWFu.Member);
		end --if
	end --if
end

function CWFu:CHAT_MSG_CHANNEL_NOTICE()
	--wenn der spieler einen neuen channel betritt wird dieser gleich aufgenommen
	--und falls ein channel verlassen wird, so wird dieser auch wieder aus der liste gestrichen

	if (arg7 == 0) then
		local temp_chan = string.upper(arg9);
		if (arg1 == "YOU_JOINED") then
			CWFu.Channel[temp_chan] = {name = arg9, number = arg8, watch = false, Members = {}};
			CWFu:UpdateChannelStatus();
		elseif (arg1 == "YOU_LEFT") then
			if (CWFu.Channel[temp_chan]) then
				CWFu.Channel[temp_chan] = "CWFu_Recycle";
				CWFu.Channel = CWFu.TableRecycle(CWFu.Channel);
			end --if
		end --if
		self:Update();
	end --if
end

function CWFu.TableRecycle(table)
	local newtable = {};
	for n,m in pairs(table) do
		if (m ~= "CWFu_Recycle") then
			newtable[n] = m;
		end --if
	end --for
	return newtable;
end

function CWFu:ChannelsUpdate()
	--ist nur dafür falls mal keine Einträge in ChannelList sind
	--übernimmt nur channels die nicht in als ServerChannels markiert sind
	
	if (GetTime()*1000 - CWFu.lastUpdate > 10000) then
		CWFu.Channel = {};

		CWFu.lastUpdate = GetTime() * 1000;
		local ChanlistL = {GetChannelList()};
		local ChanlistS = {EnumerateServerChannels()};
		local is_server;
	
		for n=1, #ChanlistL, 2 do
			is_server = false;
			for m=1, #ChanlistS do
				if (ChanlistL[n+1] == ChanlistS[m]) then
					is_server = true;
				end --if
			end --for m
			
			if (not is_server) then
				local temp_chan = string.upper(ChanlistL[n+1]);
				CWFu.Channel[temp_chan] = {name = ChanlistL[n+1], number = ChanlistL[n], watch = false, Members = {}};
			end --if
		end --for n
	
		self:UpdateChannelStatus();
	end --if
end

function CWFu:UpdateChannelStatus()
	--für alle Channels die in self.db.profile.channellist eingetragen sind
	--wird das .watch flag gesetzt und alle members aus dem channel abgerufen

	for _,n in ipairs(self.db.profile.channellist) do
		local temp_chan = string.upper(n);
		if (CWFu.Channel[temp_chan]) then
			if (not CWFu.Channel[temp_chan].watch) then
				CWFu.Channel[temp_chan].watch = true;
				CWFu:GetMembers(temp_chan);
			end --if
		end --if
	end --for n
end

function CWFu:CHAT_MSG_CHANNEL_JOIN()
	--wenn spieler einen überwachten channel betritt dann füge ihn einzigartig zu Members hinzu
	
	local temp_chan = string.upper(arg9);
	
	if (CWFu.Channel[temp_chan]) then
		if (CWFu.Channel[temp_chan].watch) then
			if (not CWFu.MemberInChannel(temp_chan, arg2)) then
				table.insert(CWFu.Channel[temp_chan].Members, arg2);
			end --if
			
			if (not CWFu.Member[arg2]) then
				CWFu.Member[arg2] = {name=arg2, level=0, class=L["N/A"], zone=L["N/A"], guild=L["N/A"], notes=L["N/A"]};
				CWFu:UserInfo(arg2, {callback = 'UserDataReturned', flags = self.WHOLIB_FLAG_ALLWAYS_CALLBACK});
			end --if
			table.sort(CWFu.Channel[temp_chan].Members);
			CWFu:Update();
		end --if
	end --if
end

function CWFu:CHAT_MSG_CHANNEL_LEAVE()
	--wenn spieler in überwachtem Channel war dann entferne ihn aus der Members liste
	local temp_chan = string.upper(arg9);
	
	--self:Print("Player (" .. arg2 .. ") left channel (" .. arg9 .. ")");
	if (CWFu.Channel[temp_chan]) then
		for n,m in ipairs(CWFu.Channel[temp_chan].Members) do
			if (m == arg2) then
				table.remove(CWFu.Channel[temp_chan].Members, n);
				CWFu:Update();
			end --if
		end --if
	end --if
end

function CWFu.MemberInChannel(channel, member)
	if (CWFu.Channel[channel]) then
		for _,n in ipairs(CWFu.Channel[channel].Members) do
			if (n == member) then
				return true;
			end --if
		end --for n
	end --if
	return false;
end

function CWFu:CHAT_MSG_CHANNEL_LIST()
	--wird ausgeführt wenn die gesammte Liste der Spieler in einem Channel aufgerufen wird
	--alle spieler werden dann zu der Channel.MemberList hinzugefügt
	--und es wird wenn nötig auch ein Eintrag in der globalen Member Liste erstellt
	
	local temp_i = string.find(arg4, ".", 1, true);
	local temp_chan = string.upper(string.sub(arg4, temp_i+2));

	--self:Print("listing: " .. temp_chan);
	if (CWFu.ScanChannel[temp_chan]) then
		--self:Print("scanning: " .. temp_chan .. " = " .. arg1);
		arg1 = string.gsub(arg1, "*", "");
	
		while #arg1 > 0 do
			local n = string.find(arg1, ",");
			if (n~=nil) then
				temp_name = string.sub(arg1, 1, n - 1);
				arg1 = string.sub(arg1, n + 2);
			else --if    
				temp_name = arg1;
				arg1 = "";
			end --if
			
			if (not CWFu.MemberInChannel(temp_chan, temp_name)) then
				table.insert(CWFu.Channel[temp_chan].Members, temp_name);
			end --if
			
			if (not CWFu.Member[temp_name]) then
				CWFu.Member[temp_name] = {name=temp_name, level=0, class=L["N/A"], zone=L["N/A"], guild=L["N/A"], notes=L["N/A"]};
				CWFu:UserInfo(temp_name, {callback = 'UserDataReturned', flags = self.WHOLIB_FLAG_ALLWAYS_CALLBACK, queue = self.WHOLIB_QUEUE_SCANNING});
			end --if
		end --while
		
		table.sort(CWFu.Channel[temp_chan].Members);
		CWFu:Update();
	else
		table.insert(CWFu.QueuedChannelInfo, temp_chan);
	end --if
	
end